package sun.security.jgss.wrapper;

import org.ietf.jgss.MessageProp;
import org.ietf.jgss.ChannelBinding;
import org.ietf.jgss.GSSException;
import java.util.Hashtable;
import org.ietf.jgss.Oid;

class GSSLibStub
{
    private Oid mech;
    private long pMech;
    private static Hashtable<Oid, GSSLibStub> table;
    
    static native boolean init(String p0, boolean p1);
    
    private static native long getMechPtr(byte[] p0);
    
    static native Oid[] indicateMechs();
    
    native Oid[] inquireNamesForMech() throws GSSException;
    
    native void releaseName(long p0);
    
    native long importName(byte[] p0, Oid p1);
    
    native boolean compareName(long p0, long p1);
    
    native long canonicalizeName(long p0);
    
    native byte[] exportName(long p0) throws GSSException;
    
    native Object[] displayName(long p0) throws GSSException;
    
    native long acquireCred(long p0, int p1, int p2) throws GSSException;
    
    native long releaseCred(long p0);
    
    native long getCredName(long p0);
    
    native int getCredTime(long p0);
    
    native int getCredUsage(long p0);
    
    native NativeGSSContext importContext(byte[] p0);
    
    native byte[] initContext(long p0, long p1, ChannelBinding p2, byte[] p3, NativeGSSContext p4);
    
    native byte[] acceptContext(long p0, ChannelBinding p1, byte[] p2, NativeGSSContext p3);
    
    native long[] inquireContext(long p0);
    
    native Oid getContextMech(long p0);
    
    native long getContextName(long p0, boolean p1);
    
    native int getContextTime(long p0);
    
    native long deleteContext(long p0);
    
    native int wrapSizeLimit(long p0, int p1, int p2, int p3);
    
    native byte[] exportContext(long p0);
    
    native byte[] getMic(long p0, int p1, byte[] p2);
    
    native void verifyMic(long p0, byte[] p1, byte[] p2, MessageProp p3);
    
    native byte[] wrap(long p0, byte[] p1, MessageProp p2);
    
    native byte[] unwrap(long p0, byte[] p1, MessageProp p2);
    
    static GSSLibStub getInstance(Oid oid) throws GSSException {
        GSSLibStub value = (GSSLibStub)GSSLibStub.table.get(oid);
        if (value == null) {
            value = new GSSLibStub(oid);
            GSSLibStub.table.put(oid, value);
            return value;
        }
        return value;
    }
    
    private GSSLibStub(Oid oid) throws GSSException {
        SunNativeProvider.debug("Created GSSLibStub for mech " + oid);
        this.mech = oid;
        this.pMech = getMechPtr(oid.getDER());
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o instanceof GSSLibStub && this.mech.equals(((GSSLibStub)o).getMech()));
    }
    
    @Override
    public int hashCode() {
        return this.mech.hashCode();
    }
    
    Oid getMech() {
        return this.mech;
    }
    
    static {
        GSSLibStub.table = new Hashtable(5);
    }
}
